#!/usr/bin/env python3

import colorsys
from math import isclose
from multiprocessing import Process


def mp_worker(color_model):
	if color_model == 'yiq':
		fun1 = colorsys.rgb_to_yiq
		fun2 = colorsys.yiq_to_rgb
	elif color_model == 'hls':
		fun1 = colorsys.rgb_to_hls
		fun2 = colorsys.hls_to_rgb
	elif color_model == 'hsv':
		fun1 = colorsys.rgb_to_hsv
		fun2 = colorsys.hsv_to_rgb
	
	for r in range(301):
		r /= 300
		print('{} r={}'.format(color_model, r))
		for g in range(301):
			g /= 300
			for b in range(301):
				b /= 300
				cv1, cv2, cv3 = fun1(r, g, b)
				r2, g2, b2 = fun2(cv1, cv2, cv3)
				if not isclose(r, r2, abs_tol=0.0001) \
				or not isclose(g, g2, abs_tol=0.0001) \
				or not isclose(b, b2, abs_tol=0.0001):
					raise Exception('using {}, rgb {},{},{} became {},{},{}'.format(
						color_model, r, g, b, r2, g2, b2))
	
	print('{} ok'.format(color_model))


def main():
	procs = []
	for cm in 'yiq', 'hls', 'hsv':
		p = Process(target=mp_worker, args=(cm,))
		p.start()
		procs.append(p)
	
	for p in procs:
		p.join()


if __name__ == '__main__':
	main()
